#include "uwbgl_Camera2.h"
#include "uwbgl_PrimitiveLine5.h"

void UWB_Camera::Draw( eLevelOfDetail lod, UWB_DrawHelper& draw_helper, float aspect_ratio ) const
{
  // draw line from the camera position to the target position (look at)
  draw_helper.SetShadeMode(smFlat);
  draw_helper.SetColor1( UWB_Color(0,0,0) );
  draw_helper.EnableTexture(false);
  draw_helper.DrawLine( m_CameraPos, m_LookAt );

  // only draw the rest if we want a high level of detail camera draw
  if( lodHigh != lod || aspect_ratio <= 0.0f )
    return;

  vec3 toward = m_LookAt - m_CameraPos;
  normalize(toward);
  vec3 up = m_UpVector;
  normalize(up);
  vec3 across = cross(toward, up);
  up = cross(across,toward);

  vec3 near_plane_center = m_CameraPos + toward * m_ViewNearPlane;
  vec3 far_plane_center = m_CameraPos + toward * m_ViewFarPlane;

  float w = ::atan( UWB_ToRadians(m_ViewFOV)) * m_ViewNearPlane;
  float h = w / aspect_ratio;

  vec3 near_corners[4];
  near_corners[0] = near_plane_center - across * w - up * h;
  near_corners[1] = near_plane_center + across * w - up * h;
  near_corners[2] = near_plane_center + across * w + up * h;
  near_corners[3] = near_plane_center - across * w + up * h;

  w = ::atan( UWB_ToRadians(m_ViewFOV)) * m_ViewFarPlane;
  h = w / aspect_ratio;

  vec3 far_corners[4];
  far_corners[0] = far_plane_center - across * w - up * h;
  far_corners[1] = far_plane_center + across * w - up * h;
  far_corners[2] = far_plane_center + across * w + up * h;
  far_corners[3] = far_plane_center - across * w + up * h;

  // draw white lines from location to the near plane
  draw_helper.SetColor1( UWB_Color(255,255,255) );
  draw_helper.DrawLine( m_CameraPos, near_corners[0] );
  draw_helper.DrawLine( m_CameraPos, near_corners[1] );
  draw_helper.DrawLine( m_CameraPos, near_corners[2] );
  draw_helper.DrawLine( m_CameraPos, near_corners[3] );

  // draw black lines defining the view frustum
  draw_helper.SetColor1( UWB_Color(0,0,0) );
  draw_helper.DrawLine( near_corners[0], near_corners[1] );
  draw_helper.DrawLine( near_corners[1], near_corners[2] );
  draw_helper.DrawLine( near_corners[2], near_corners[3] );
  draw_helper.DrawLine( near_corners[3], near_corners[0] );

  draw_helper.DrawLine( far_corners[0], far_corners[1] );
  draw_helper.DrawLine( far_corners[1], far_corners[2] );
  draw_helper.DrawLine( far_corners[2], far_corners[3] );
  draw_helper.DrawLine( far_corners[3], far_corners[0] );

  draw_helper.DrawLine( near_corners[0], far_corners[0] );
  draw_helper.DrawLine( near_corners[1], far_corners[1] );
  draw_helper.DrawLine( near_corners[2], far_corners[2] );
  draw_helper.DrawLine( near_corners[3], far_corners[3] );
}
