#include "uwbgl_PrimitiveLine5.h"

UWB_PrimitiveLine::UWB_PrimitiveLine()
: m_start(0,0,0)
, m_end(0,0,0)
{
}

void UWB_PrimitiveLine::SetupDrawAttributes(UWB_DrawHelper& draw_helper)
{
  __super::SetupDrawAttributes(draw_helper);
  draw_helper.SetColor2(m_ShadingColor);
  draw_helper.SetShadeMode(m_ShadeMode);
}

void UWB_PrimitiveLine::DrawPrimitive(eLevelOfDetail lod, UWB_DrawHelper& draw_helper)
{
  draw_helper.DrawLine(m_start, m_end);
}

void UWB_PrimitiveLine::MouseDownVertex(int vertexID, float x, float y)
{
  if(0 == vertexID)
    SetStartPoint(x,y);

  SetEndPoint(x,y);
}

void UWB_PrimitiveLine::MoveTo(float x, float y)
{
  SetEndPoint(x,y);
}

vec3 UWB_PrimitiveLine::GetLocation() const
{
  return GetMidPoint();
}

void UWB_PrimitiveLine::SetStartPoint(float x, float y, float z)
{
  m_start = vec3(x,y,z);
}

vec3 UWB_PrimitiveLine::GetStartPoint() const
{
  return m_start;
}

void UWB_PrimitiveLine::SetEndPoint(float x, float y, float z)
{
  m_end = vec3(x,y,z);
}

vec3 UWB_PrimitiveLine::GetEndPoint() const
{
  return m_end;
}

vec3 UWB_PrimitiveLine::GetMidPoint() const
{
  vec3 m = (m_start + m_end) / 2.0f;
  return m;
}

void UWB_PrimitiveLine::Update(float elapsed_seconds)
{
  // current position is defined by previous position + velocity * time
  vec3 adjust_vec = m_velocity * elapsed_seconds;
  // to move a line, we need to move both the start and end points
  m_start += adjust_vec;
  m_end += adjust_vec;
}

const UWB_BoundingVolume* UWB_PrimitiveLine::GetBoundingVolume( eLevelOfDetail lod ) const
{
  //lazy update for bounding box
  {
    UWB_PrimitiveLine* pLine = const_cast<UWB_PrimitiveLine*>(this);
    pLine->m_bounds.SetCorners(m_start, m_end);
  }

  return &m_bounds;
}
