// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_Primitive5.h"

///
/// A primitive drawing object that represents a line
class UWB_PrimitiveLine : public UWB_Primitive
{
public:
  /// Initialize the start and end points to {0,0,0}
  /// The default constructor creates a zero length line
  UWB_PrimitiveLine();

  /// Update the position of the line based in it's velocity
  /// \param elapsed_seconds amount of time that has passed since the last update
  void Update(float elapsed_seconds);

  /// User mouse input handler for this primitive
  /// position is set to the given mouse input location
  void MouseDownVertex(int vertexID, float x, float y);

  /// Mouse user input handler for this primitive
  /// Move the position of the point to a new location
  void MoveTo(float x, float y);

  /// Get the current location of this line in 3D coordinates
  /// This is considered the midpoint of the line
  vec3 GetLocation() const;

  /// Virtual override for getting the bounding volume of this primitive
  /// This always just returns the address of m_bounds
  virtual const UWB_BoundingVolume* GetBoundingVolume( eLevelOfDetail lod ) const;

  void SetStartPoint(float x, float y, float z=0.0f);
  vec3 GetStartPoint() const;
  void SetEndPoint(float x, float y, float z=0.0f);
  vec3 GetEndPoint() const;

  /// Return the position in space that is half way between the start
  /// and end points of this line
  vec3 GetMidPoint() const;

protected:
  /// Virtual drawing routine for this line. All of the drawing operations
  /// are called on the draw_helper in order to abstract the graphics API from
  /// the primitive class
  void DrawPrimitive( eLevelOfDetail lod, UWB_DrawHelper& draw_helper );

  /// Virtual function used to define the current attributes for drawing
  /// \param draw_helper this object is first passed to this function for setting
  ///        up it's attribute state and then the DrawPrimitive function is called
  ///        with this same object
  void SetupDrawAttributes(UWB_DrawHelper& draw_helper);

protected:
  ///
  /// The start position of this line
  vec3 m_start;
  ///
  /// The end position of this line
  vec3 m_end;
  UWB_BoundingBox m_bounds;
};
