#include "uwbgl_PrimitiveList4.h"

UWB_PrimitiveList::UWB_PrimitiveList()
{
}

void UWB_PrimitiveList::SetupDrawAttributes(UWB_DrawHelper& draw_helper)
{
  // don't need to, and don't want to do anything
}

void UWB_PrimitiveList::DrawPrimitive(eLevelOfDetail lod, UWB_DrawHelper& draw_helper)
{
  int count = m_list.Count();
  for(int i=0; i<count; i++)
  {
    UWB_Primitive* pPrimitive = m_list[i];
    if(pPrimitive)
      pPrimitive->Draw(lod, draw_helper);
  }
}

void UWB_PrimitiveList::Update(float elapsed_seconds)
{
  int count = m_list.Count();
  for(int i=0; i<count; i++)
  {
    UWB_Primitive* pPrimitive = m_list[i];
    if(pPrimitive)
      pPrimitive->Update(elapsed_seconds);
  }
}

const UWB_BoundingVolume* UWB_PrimitiveList::GetBoundingVolume( eLevelOfDetail lod) const
{
  int count = m_list.Count();
  if(0 == count)
    return NULL;

  {
    // NOTE: This is pretty inefficient. We could speed this up by maintaining
    // some sort of "changed" flag and only updating the bounds when this flag changes
    UWB_PrimitiveList* pList = const_cast<UWB_PrimitiveList*>(this);
    pList->m_bounds.MakeInvalid();

    for(int i=0; i<count; i++)
    {
      UWB_Primitive* pChild = pList->m_list[i];
      if(pChild)
      {
        const UWB_BoundingVolume* pBV = pChild->GetBoundingVolume(lod);
        pList->m_bounds.Add(pBV);
      }
    }
  }

  return &m_bounds;
}

int UWB_PrimitiveList::Count() const
{
  return m_list.Count();
}

UWB_Primitive* UWB_PrimitiveList::PrimitiveAt(int index)
{
  if(index<0 || index>=m_list.Count())
    return NULL;
  return m_list[index];
}

const UWB_Primitive* UWB_PrimitiveList::PrimitiveAt(int index) const
{
  if(index<0 || index>=m_list.Count())
    return NULL;
  return m_list[index];
}

void UWB_PrimitiveList::Append(UWB_Primitive* pPrimitive)
{
  m_list.Append(pPrimitive);
}

void UWB_PrimitiveList::DeletePrimitiveAt(int index)
{
  m_list.DeleteItem(index);
}

void UWB_PrimitiveList::DrawChildBoundingVolumes(eLevelOfDetail lod, UWB_DrawHelper* pDrawHelper, UWB_Color color) const
{
  if(NULL == pDrawHelper)
    return;
  int count = Count();
  for(int i=0; i<count; i++)
  {
    const UWB_Primitive* pPrimitive = m_list[i];
    if(pPrimitive)
      pPrimitive->DrawBoundingVolume(lod, pDrawHelper, color);
  } 
}
