#include "uwbgl_XformInfo5.h"

UWB_XformInfo::UWB_XformInfo()
: m_pivot(0,0,0)
{
  ToIdentity();
}

void UWB_XformInfo::SetUpModelStack(UWB_DrawHelper& draw_helper) const
{
  mat3 m = GetRotation();
  draw_helper.AccumulateModelTransform( m_translation, m_scale, m, m_pivot );
}

void UWB_XformInfo::ToIdentity()
{
  m_translation = vec3(0.0f,0.0f,0.0f);
  m_scale = vec3(1.0f,1.0f,1.0f);
  InitializeRotation();
  // pivot is not set
}

void UWB_XformInfo::DrawPivot(UWB_DrawHelper &h, float size) const
{
  h.ResetAttributes();
  h.SetColor1(UWB_Color(1.0f, 1.0f, 1.0f) );
  h.SetColor2(UWB_Color(0.0f, 0.0f, 1.0f) );
  h.SetFillMode( fmSolid );
  h.SetShadeMode( smGouraud );
  h.DrawCircle( m_pivot, size);
}

vec3 UWB_XformInfo::GetTranslation() const{ return m_translation; }
void UWB_XformInfo::SetTranslation(vec3 translation){ m_translation = translation; }

vec3 UWB_XformInfo::GetScale() const {return m_scale;}
void UWB_XformInfo::SetScale(vec3 scale) { m_scale = scale; }

void UWB_XformInfo::SetRotation(const mat3 &m)
{
  mat3 r(m);
  makeTranspose(r);
  m_rotation = mat2quat(r);
}

mat3 UWB_XformInfo::GetRotation() const
{
  mat3 r = quat2mat(m_rotation);
  makeTranspose(r);
  return r;
}

void UWB_XformInfo::SetRotationQuat(const quat &q)
{
  m_rotation = q;
}

quat UWB_XformInfo::GetRotationQuat() const
{
  return m_rotation;
}



void UWB_XformInfo::InitializeRotation()
{
  m_rotation[0] = m_rotation[1] = m_rotation[2] = 0.0f;
  m_rotation[3] = 1.0f; // this is w
}
void UWB_XformInfo::UpdateRotationXByRadian(float dx)
{
  UpdateRotationXByDegree( UWB_ToDegrees(dx) );
}
void UWB_XformInfo::UpdateRotationYByRadian(float dy)
{
  UpdateRotationYByDegree( UWB_ToDegrees(dy) ); 
}
void UWB_XformInfo::UpdateRotationZByRadian(float dz)
{
  UpdateRotationZByDegree( UWB_ToDegrees(dz) );
}

void UWB_XformInfo::UpdateRotationXByDegree(float dx)
{
  quat q = quatFromAA(vec3(1,0,0),dx);
  m_rotation = m_rotation * q;
}

void UWB_XformInfo::UpdateRotationYByDegree(float dy)
{
  quat q = quatFromAA(vec3(0,1,0),dy);
  m_rotation = m_rotation * q;
}

void UWB_XformInfo::UpdateRotationZByDegree(float dz)
{
  quat q = quatFromAA(vec3(0,0,1),dz);
  m_rotation = m_rotation * q;
}

void UWB_XformInfo::UpdateRotationByQuat(const quat &q)
{
  m_rotation = q * m_rotation;
}

vec3 UWB_XformInfo::GetPivot() const {return m_pivot;}
void UWB_XformInfo::SetPivot(vec3 pivot) {m_pivot = pivot;}